import { useFormik } from 'formik';
import Button from '@material-ui/core/Button';
import TextField from '@material-ui/core/TextField';
import { connect } from 'react-redux';

import { validationSchemaRegistration } from '../helpers/validation';
import togglePassword from '../helpers/helpers';
import { asyncCreateUser } from '../../../redux/authorization/operation';
import {
  IRegistrationProps,
  IRegistrationState,
} from '../../../typescript/components/Registration/interfaces';
import s from './RegistrationForm.module.css';

const RegistrationForm = ({ isLoading, createUser }: IRegistrationProps) => {
  const formik = useFormik({
    initialValues: {
      email: '',
      password: '',
    },
    validationSchema: validationSchemaRegistration,
    onSubmit: ({ email: login, password }) => createUser(login, password),
  });

  return (
    <div>
      <form className={s.registration_form} onSubmit={formik.handleSubmit}>
        <div className={s.registration_input_wrapper}>
          <TextField
            id="email"
            name="email"
            label="Email"
            value={formik.values.email}
            onChange={formik.handleChange}
            error={formik.touched.email && Boolean(formik.errors.email)}
            helperText={formik.touched.email && formik.errors.email}
          />
          <TextField
            id="password"
            name="password"
            label="Password*"
            type="password"
            value={formik.values.password}
            onChange={formik.handleChange}
            error={formik.touched.password && Boolean(formik.errors.password)}
            helperText={formik.touched.password && formik.errors.password}
          />
          <div>
            <input
              className={s.registration_show_password}
              type="checkbox"
              onClick={togglePassword}
            />
            Show password*
          </div>
        </div>
        <Button
          className={s.registration_button}
          color="primary"
          variant="contained"
          disabled={isLoading ? true : false}
          type="submit"
        >
          Submit
        </Button>
      </form>
    </div>
  );
};

const mapStateToProps = (state: IRegistrationState) => ({
  isLoading: state.isLoading.flag,
});

const mapDispatchToProps = (
  dispatch: (dispatchAction: any) => Promise<void>,
) => ({
  createUser: (login: string, password: string) =>
    dispatch(asyncCreateUser(login, password)),
});

export default connect(mapStateToProps, mapDispatchToProps)(RegistrationForm);
