import { useEffect, useState } from 'react';
import { connect } from 'react-redux';
import { useRouteMatch } from 'react-router-dom';

import s from './DetailGood.module.css';
import { goodById } from '../../../api-data';
import { asyncMakeOrderById } from '../../../redux/user/operation';
import { IGoodDetail } from '../../../typescript/components/Goods/interfaces';
import {
  TRouteMatchId,
  TDispatchLoading,
} from '../../../typescript/components/Goods/types';
import { actionLoading } from '../../../redux/loading/action';

function DetailGood({ dispatchLoading }: TDispatchLoading) {
  const {
    params: { id },
  }: TRouteMatchId = useRouteMatch();
  const randomPrice = (min: number, max: number) =>
    Math.floor(Math.random() * (max - min + 1) + min);
  const price = randomPrice(1, 10);
  const handlePurchase = async () => asyncMakeOrderById(price, id);
  const [good, setGood] = useState<IGoodDetail | null>(null);

  useEffect(() => {
    async function getGood() {
      try {
        dispatchLoading(true);
        const data = await goodById<IGoodDetail>(id);
        setGood(data);
      } catch (e) {
        console.error(e);
      } finally {
        dispatchLoading(false);
      }
    }
    getGood();
  }, [dispatchLoading, id]);

  return good ? (
    <div className={s.detailGood_wrapper}>
      <p>name : {good.name ? good.name : 'missed'}</p>
      <p>createdAt : {good.createdAt ? good.createdAt : 'missed'}</p>
      <p>fake price for redux : {price}</p>
      <p>price : {good.price ? good.price : 'missed'}</p>
      <p>description : {good.description ? good.description : 'missed'}</p>
      <img
        src={`http://shop-roles.asmer.fs.a-level.com.ua/${
          good.images && good.images[0].url
        }`}
        alt="*it has to be pic of product*"
        width="200"
        height="200"
      ></img>
      <button
        onClick={handlePurchase}
        className={s.detailGoodBtn}
        type="button"
      >
        Purchase
      </button>
    </div>
  ) : null;
}

const mapDispatchToProps = (
  dispatch: (dispatchAction: any) => Promise<void>,
) => ({
  dispatchLoading: (value: boolean) => dispatch(actionLoading(value)),
});

export default connect(null, mapDispatchToProps)(DetailGood);
