import { useEffect, useState } from 'react';
import { Route } from 'react-router-dom';
import { connect } from 'react-redux';

import s from './Orders.module.css';
import { asyncGetOrders } from '../../redux/orders/operation';
import {
  IOrdersProps,
  IOrdersState,
} from '../../typescript/components/Orders/interfaces';

import Order from './Order/Order';
import Pagination from '../PaginationBar/PaginationBar';

const Orders = ({ ordersArr, getOrders }: IOrdersProps) => {
  const [page, setPage] = useState<number>(1);

  const handlePageChange = (page: number) => setPage(page);

  useEffect(() => {
    getOrders();
  }, [getOrders]);

  return ordersArr ? (
    <Route>
      <ul className={s.orders_list}>
        {ordersArr.length !== 0 &&
          ordersArr
            .slice((page - 1) * 5, page * 5)
            .map((order, i: number) => <Order key={i} order={order} />)}
      </ul>
      <Pagination
        page={page}
        total={Math.ceil(ordersArr.length / 5)}
        handlePageChange={handlePageChange}
      />
    </Route>
  ) : null;
};

const mapStateToProps = (state: IOrdersState) => ({
  ordersArr: state.orders.ordersArr,
});

const mapDispatchToProps = (
  dispatch: (dispatchAction: any) => Promise<void>,
) => ({
  getOrders: () => dispatch(asyncGetOrders()),
});

export default connect(mapStateToProps, mapDispatchToProps)(Orders);
